
//Import the following:
#include <ESP8266WiFi.h>
#include <PubSubClient.h>

#include <NTPClient.h>
#include <WiFiUdp.h>

// setup for servo
#include <Servo.h>
Servo servoTop1;
Servo servoBottom1;
Servo servoTop2;
Servo servoBottom2;

// define the angles of each mode
#define closeBottomAngle 0
#define openBottomAngle 90 
#define closeTopAngle 90
#define openTopAngle 0 

// setup for pills
int pillAmount1 = 0;
int pillAmount2 = 0;
int wantedHour = 0;
int wantedMinute = 0;

// define the amount of time (in seconds) for the pills to be taken before notification, 600 = 10 minutes
#define takeWindow 10 //<----------------------------------------------------------------------------------- can be updated before use

// open a HotSpot from your PC and enter the credentials below
const char* ssid = "DEKEL-HOTSPOT"; //<--------------------------------------------------------------------- needs to be updated before use
const char* password = "12345678"; //<---------------------------------------------------------------------- needs to be updated before use

// run Mosquitto, check the HotSpot IP with the command "ifconfig" in the cmd and enter it below
const char* mqtt_server = "192.168.137.1";//<--------------------------------------------------------------- needs to be updated before use
 
// initializes the espClient
WiFiClient espClient;
PubSubClient client(espClient);

// change utcOffsetInSeconds to account for your timezone, example: For UTC +3.00 : 3 * 60 * 60 : 10800
const long utcOffsetInSeconds = 10800; //<------------------------------------------------------------------- needs to be updated before use
char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
// define NTP Client to get time
WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, "pool.ntp.org", utcOffsetInSeconds);

// set pins for tray
#define trayPin 14 // GPIO14 which is D5
int trayStatus = 0;

// connects your ESP8266 to your router
void setup_wifi() {
  delay(10);
  Serial.begin(9600);
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("WiFi connected - ESP IP address: ");
  Serial.println(WiFi.localIP());
}

// this function reconnects your ESP8266 to your MQTT broker
void reconnect() {
  // loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    if (client.connect("ESP8266Client")) {
      Serial.println("connected");  
      client.subscribe("dispenser/#");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

// starts wifi communication, connects to the mqtt server, connects to the NTP server to recieve the time, sets the pins for the tray and servos
void setup() {
  pinMode(trayPin, INPUT_PULLUP);
  Serial.begin(9600);
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.subscribe("dispenser/#");
  client.setCallback(callback);
  timeClient.begin();

  // attach servo pins and start all in closed position
  servoTop1.attach(4); // GPIO4 which is pin D2
  servoTop1.write(closeTopAngle); 
  servoBottom1.attach(2); // GPIO2 which is pin D4
  servoBottom1.write(closeBottomAngle); 
  servoTop2.attach(0); // GPIO0 which is pin D3
  servoTop2.write(closeTopAngle);
  servoBottom2.attach(5); // GPIO5 which is pin D1
  servoBottom2.write(closeBottomAngle);
}

// ensures that ESP is connected to your broker
void loop() {
  if (!client.connected()) {
    reconnect();
  }
  if(!client.loop()) {
    client.connect("ESP8266Client");
  }
  dispenserCheck();
} 

void dispenserCheck(){
  // update current time
  timeClient.update();
  int currentHour = timeClient.getHours();
  int currentMinute = timeClient.getMinutes();
  int currentSecond = timeClient.getSeconds();
  
  // check if the requested time is now
  if((wantedHour == currentHour) && (wantedMinute == currentMinute) && (currentSecond == 0)){
    for(int i = 0; i < pillAmount1; i++){
      servoTop1.write(openTopAngle);
      delay(1000);
      servoTop1.write(closeTopAngle);
      delay(1000);
      servoBottom1.write(openBottomAngle);
      delay(500);
      servoBottom1.write(closeBottomAngle);
      delay(500);
    }
    for(int i = 0; i < pillAmount2; i++){
      servoTop2.write(openTopAngle);
      delay(2000);
      servoTop2.write(closeTopAngle);
      delay(2000);
      servoBottom2.write(openBottomAngle);
      delay(500);
      servoBottom2.write(closeBottomAngle);
      delay(500);
    }
    trayCheck();
  }
  
}

// check if the pills where taken within the given window
void trayCheck(){
  trayStatus = digitalRead(trayPin);
  int count = 0;
  boolean taken = false;
  while((trayStatus == 0) && (count < takeWindow)){
    delay(1000);
    count++;
    trayStatus = digitalRead(trayPin);
    Serial.println(trayStatus);// for debugging
    if(trayStatus == 1){
      taken = true;
      break;
    }
  }
  if(!taken){
    if (!client.connected()) {
      reconnect();
    }
    client.publish("dispenser/tray","inform");
  }

}

// This function is executed when some device publishes a message to a topic that your ESP8266 is subscribed to
void callback(String topic, byte* message, unsigned int length) {
  Serial.print("Message arrived on topic: ");
  Serial.print(topic);
  Serial.print(". Message: ");
  String messageTemp;
  for (int i = 0; i < length; i++) {
    Serial.print((char)message[i]);
    messageTemp += (char)message[i];
  }
  Serial.println();

  // update amount of pill 1 to dispense
  if(topic=="dispenser/pill1"){
    pillAmount1 = messageTemp.toInt();
  }
  // update amount of pill 2 to dispense
  if(topic=="dispenser/pill2"){
    pillAmount2 = messageTemp.toInt();
  }
  // update dispensing timer hour 
  if(topic=="dispenser/timer/hour"){
    wantedHour = messageTemp.toInt();
  }
  // update dispensing timer minute
  if(topic=="dispenser/timer/minutes"){
    wantedMinute = messageTemp.toInt();
  }
  // dispense requested amount of pill 1 now (will not start a check and notify if not taken, this can be added if you so desire)
  if(topic=="dispenser/pillNow1"){
    for(int i = 0; i < pillAmount1; i++){
      servoTop1.write(openTopAngle);
      delay(1000);
      servoTop1.write(closeTopAngle);
      delay(1000);
      servoBottom1.write(openBottomAngle);
      delay(500);
      servoBottom1.write(closeBottomAngle);
      delay(500);
    }
  }
  // dispense requested amount of pill 2 now (will not start a check and notify if not taken, this can be added if you so desire)
  if(topic=="dispenser/pillNow2"){
    for(int i = 0; i < pillAmount2; i++){
      servoTop2.write(openTopAngle);
      delay(2000);
      servoTop2.write(closeTopAngle);
      delay(2000);
      servoBottom2.write(openBottomAngle);
      delay(500);
      servoBottom2.write(closeBottomAngle);
      delay(500);
    }
  }
  
  // debug - opens and closes each servo with the manual switchs (under debug GUI)
  if(topic=="dispenser/debug/top1"){
    if(messageTemp == "true"){
      servoTop1.write(openTopAngle);
    }
    if(messageTemp == "false"){
      servoTop1.write(closeTopAngle);
    }
  }
  if(topic=="dispenser/debug/bottom1"){
    if(messageTemp == "true"){
      servoBottom1.write(openBottomAngle);
    }
    if(messageTemp == "false"){
      servoBottom1.write(closeBottomAngle);
    }
  }
   if(topic=="dispenser/debug/top2"){
    if(messageTemp == "true"){
      servoTop2.write(openTopAngle);
    }
    if(messageTemp == "false"){
      servoTop2.write(closeTopAngle);
    }
  }
  if(topic=="dispenser/debug/bottom2"){
    if(messageTemp == "true"){
      servoBottom2.write(openBottomAngle);
    }
    if(messageTemp == "false"){
      servoBottom2.write(closeBottomAngle);
    }
  }
  }
